/*------------------------------------------------------------------------------------
   Name: xOpenCloseRange.mq4
   Copyright 2010, Xaphod, http://forexwhiz.appspot.com
   
   Description: Draw lines to show the candle open/close range
	          
   Change log: 
       2010-06-28. Xaphod, v1.0
          - First Release 
-------------------------------------------------------------------------------------*/
// Indicator properties
#property copyright "Copyright  2010, Xaphod"
#property link      "http://forexwhiz.appspot.com"

#property indicator_chart_window
#property indicator_buffers 2
//#include <Xi-Debug.mqh>

// Constant definitions
#define INDICATOR_NAME "xOpenCloseRange"
#define INDICATOR_VERSION "1.0"


// Indicator parameters
extern string    Copyright="2010, forexwhiz.appspot.com";
extern string    Version= INDICATOR_VERSION;
extern string    Settings="";
extern string    SetupBarTime="09:00"; // Setup candle time. It must be exact for the time frame. Ex 09:15 will work on M15, 09:16 will not
extern string    SessionName="LO";     // Name of the session. Change if you use multiple instances on the same chart
extern int       SessionBars=16;       // Number of bar to plot session levels
extern bool      ShowRange=True;       // Show label with session name and range 
extern color     LineColor=Yellow;     // Level line color
extern int       LineWidth=1;          // 1-5
extern int       LineStyle=STYLE_DOT;  // 0=SOLID, 1=DASH, 2=DOT, 3=DASHDOT, 4=DASHDOTDOT
extern int       FractionalPips=0;     // 0=4 digit broker, 1=5digit broker etc

// Global module varables
double dTopLevel[];
double dBottomLevel[];
int giDigitsFactor;

//-----------------------------------------------------------------------------
// function: init()
// Description: Custom indicator initialization function.
//-----------------------------------------------------------------------------
int init() {
   SetIndexStyle(0, DRAW_LINE,LineStyle,LineWidth,LineColor);
   SetIndexBuffer(0, dTopLevel);
   SetIndexStyle(1, DRAW_LINE,LineStyle,LineWidth,LineColor);
   SetIndexBuffer(1, dBottomLevel);
   IndicatorShortName(INDICATOR_NAME+" v"+INDICATOR_VERSION);
   SetupBarTime=ValidateTime(SetupBarTime);
   SetDigits();
   // Clear indicator buffers
   for (int i=0; i<Bars; i++) {
    dTopLevel[i]=EMPTY_VALUE;
    dBottomLevel[i]=EMPTY_VALUE;
   }
  return(0);
}

//-----------------------------------------------------------------------------
// function: deinit()
// Description: Custom indicator deinitialization function.
//-----------------------------------------------------------------------------
int deinit() {
  DeleteObjects(INDICATOR_NAME+"_"+SessionName);
  return (0);
}

void SetDigits() {
  if (FractionalPips<0) FractionalPips=0;
  if (FractionalPips>0)
    giDigitsFactor=MathPow(10,FractionalPips);
  else
    giDigitsFactor=1;
}

///-----------------------------------------------------------------------------
// function: start()
// Description: Custom indicator iteration function.
//-----------------------------------------------------------------------------
int start() {
  int iNewBars;
  int iCountedBars; 
  int i;
  static double dTop;
  static double dBottom;
  static int iSessionBars=0;
  static datetime tCurTime;
  datetime tSetupTime;
  
  
  // Get unprocessed ticks
  iCountedBars=IndicatorCounted();
  if(iCountedBars < 0) return (-1); 
  if(iCountedBars>0) iCountedBars--;
  iNewBars=Bars-iCountedBars;
  
  for(i=iNewBars; i>=0; i--) {
    tSetupTime=StrToTime(TimeToStr(Time[i],TIME_DATE )+" "+SetupBarTime); 
    // Setup bar 
    if (Time[i]==tSetupTime) {
      // Set Top & Bottom
      if (Open[i]>Close[i]) {
        dTop=Open[i]; 
        dBottom=Close[i];
      }
      else {
        dBottom=Open[i]; 
        dTop=Close[i];
      }
      //Draw top and bottom
      dTopLevel[i]=dTop;
      dBottomLevel[i]=dBottom;
      dTopLevel[i+1]=dTop;
      dBottomLevel[i+1]=dBottom;
      dTopLevel[i+2]=EMPTY_VALUE;
      dBottomLevel[i+2]=EMPTY_VALUE; 
      
      iSessionBars=1;
      if (ShowRange)
        DrawRangeValue((dTop-dBottom)/(Point*giDigitsFactor), i, dBottom);
    }
    // Session Active
    else if (iSessionBars>0 && iSessionBars<SessionBars) {
      dTopLevel[i]=dTop;
      dBottomLevel[i]=dBottom;
      // new bar. Inc bar counter
      if (tCurTime!=Time[0]) {
         tCurTime=Time[i];
         iSessionBars++;
      }
      if (ShowRange)
        DrawRangeValue((dTop-dBottom)/(Point*giDigitsFactor), i+iSessionBars/2, dBottom);
    }
    // Session completed
    else {
      // Reset data
      dTopLevel[i]=EMPTY_VALUE;
      dBottomLevel[i]=EMPTY_VALUE;
      dTop=EMPTY_VALUE;
      dBottom=EMPTY_VALUE;
      iSessionBars=0;
    }
  }
  
  return(0);
}
//+------------------------------------------------------------------+


//-----------------------------------------------------------------------------
// function: ValidateTime()
// Description: Return a valid time format (ex. 08:30) or an error.
//-----------------------------------------------------------------------------
string ValidateTime(string sTime) {  
  if (StringLen(sTime)==1) return("0"+sTime+":00");
  if (StringLen(sTime)==2) return(sTime+":00");
  if (StringLen(sTime)==4 && StringSubstr(sTime, 1, 1)==":")  return("0"+sTime);
  if (StringLen(sTime)==4) return(StringSubstr(sTime, 0, 2)+":"+StringSubstr(sTime, 2, 2));
  if (StringLen(sTime)==5 && StringSubstr(sTime, 2, 1)==":") return(sTime);
  if (StringLen(sTime)==5 && StringSubstr(sTime, 2, 1)==".") return(StringSubstr(sTime, 0, 2)+":"+StringSubstr(sTime, 3, 2));
  // Error warning 
  Alert(Symbol()," Error: Invalid Time Format in input parameters! ",sTime); 
  return(sTime);
}

//-----------------------------------------------------------------------------
// function: DrawRangeValue()
// Description: Draw range text label below the high channel line
//-----------------------------------------------------------------------------
int DrawRangeValue(double dRange, int iBarIndex, double dPrice) {
  double tTextPos=0;
  string sLineId;
  string sRange;
  
  sRange=SessionName+" ("+DoubleToStr(dRange,FractionalPips)+")";
  
  
  sLineId=INDICATOR_NAME+"_"+SessionName+"_Range_"+TimeToStr(Time[iBarIndex],TIME_DATE );
  // Create object     
  if (ObjectFind(sLineId)<0 ) {
    ObjectCreate(sLineId, OBJ_TEXT, 0, Time[iBarIndex], dPrice); 
    ObjectSet(sLineId, OBJPROP_BACK, false);
  }
  // Update object
  ObjectSetText(sLineId, sRange , 8, "Arial", LineColor);
  ObjectMove(sLineId,0,Time[iBarIndex],dPrice);  
  return(0);
}

//-----------------------------------------------------------------------------
// Function: DeleteObjects()
// Description: Delete chart objects
//-----------------------------------------------------------------------------
void DeleteObjects(string sObjId) {
  // Delete chart objects
  for(int i=ObjectsTotal()-1; i>-1; i--)
    if (StringFind(ObjectName(i),sObjId)>=0)  ObjectDelete(ObjectName(i));
  return(0);
}


